<?php

/**
 * @file
 * Callback pages and forms.
 */

/**
 * Menu callback; Return a page with list of affiliate store products.
 *
 * @param string $str_tids
 *   (optional) Term ID format string of products to display. Default to empty
 *   meaning display all.
 */
function affiliate_store_page($str_tids = '') {
  $active = affiliate_store_terms_parse_string($str_tids);
  $breadcrumb = array(l(t('Home'), NULL));
  $breadcrumb[] = l(affiliate_store_title(t('Store')), 'affiliate-store');
  $browse_title = t('Category');
  $browse_items = array();
  // The category tree is used for generating browse links.
  $tree = affiliate_store_get_category_tree();

  // Store the currently selected term, its term name or synonym, and its
  // vocabulary.
  $term = NULL;
  $name = NULL;
  $vocab = NULL;
  // Generate trail from active terms.
  // Additional term IDs in URL are the first term ID group's parents.
  for ($len = count($active), $i = $len - 1; $i >= 0; --$i) {
    $term = taxonomy_get_term($active[$i][0]);
    if (!$term) {
      continue;
    }
    $vocab = affiliate_store_get_vocabulary_name($term->vid);
    if (!$vocab) {
      continue;
    }
    $name = affiliate_store_get_term_synonym($term, $term->name);

    $path = implode('+', $active[$i]);
    // Add parent paths.
    for ($j = $i + 1; $j < $len; ++$j) {
      $path .= ',' . implode('+', $active[$j]);
    }
    $breadcrumb[] = l($name, "affiliate-store/term/$path");
  }

  $result = NULL;
  // Category, subcategory, or subsubcategory term selected.
  $categories = array('category', 'subcategory', 'subsubcategory');
  if (in_array($vocab, $categories, TRUE)) {
    $browse_title = check_plain($name);
    $browse_items = _affiliate_store_select_browse_links($tree, $vocab);
    $result = _affiliate_store_select_nodes($active);
  }
  // Term other than category and subcategory has been selected, e.g. network
  // or merchant term. Display nodes the same way as taxonomy do for single
  // term.
  elseif ($term) {
    $result = _affiliate_store_select_nodes(array(array($term->tid)));
  }
  // No term selected, display all categories.
  else {
    $tids = array();
    foreach ($tree as $category) {
      $tids = array_merge($tids, $category['tids']);
      $browse_items[] = l($category['name'], $category['path']);
    }
    $tids = array_unique($tids);
    if ($tids) {
      $result = _affiliate_store_select_nodes(array($tids));
    }
  }

  drupal_set_breadcrumb($breadcrumb);

  $products = taxonomy_render_nodes($result);
  return theme(
    'affiliate_store',
    $browse_title, $browse_items, $products, $term
  );
}

/**
 * Find all active child terms of a parent term, and format them as browse
 * links.
 *
 * @param array &$tree
 *   A reference to category tree.
 * @param string $vocabulary
 *   Vocabulary logical name of parent term.
 *
 * @return array
 *   Active child terms formatted as browse links.
 *
 * @see affiliate_store_vocabularies()
 */
function _affiliate_store_select_browse_links(&$tree, $vocabulary) {
  $result = array();
  foreach ($tree as $branch) {
    if (!empty($branch['parent']['active'])
      && ($branch['parent']['vocabulary'] === $vocabulary)
    ) {
      $result[] = l($branch['name'], $branch['path']);
    }
    if (isset($branch['children'])) {
      $func = __FUNCTION__;
      $result = array_merge($result, $func($branch['children'], $vocabulary));
    }
  }
  return $result;
}

/**
 * Finds all nodes that match selected terms for used with a pager.
 *
 * Taxonomy_select_nodes() can't be used because terms might be shared by
 * different vocabularies. Parent term is required for correctly selecting
 * products.
 *
 * @param array $tids
 *   An array of term ID groups.
 * @param string $order
 *   (optional) The order clause for the query that retrieve the nodes. The
 *   field must be in the format {table_alias}.{field_name}. Available table
 *   aliases are:
 *   - n: node
 *   - p: affiliate_store_product
 *   - t: term_node
 *   Default to n.sticky DESC, n.title ASC, p.price ASC.
 * @param int $pager
 *   (optional) Integer to distinguish between multiple pagers on one page.
 *   Default to 0.
 *
 * @return
 *   A resource identifier pointing to the query results.
 *
 * @see affiliate_store_terms_parse_string()
 */
function _affiliate_store_select_nodes($tids,
$order = 'n.sticky DESC, n.title ASC, p.price ASC', $pager = 0) {
  $join = array('JOIN {affiliate_store_product} AS p ON p.vid = n.vid');
  $where = array('n.status = %d', "n.type = '%s'");
  $args = array(1, 'affiliate_store_product');
  foreach ($tids as $index => $group) {
    $join[] = "JOIN {term_node} AS t$index ON t$index.vid = n.vid";
    $where[] = "t$index.tid IN (" . db_placeholders($group, 'int') . ")";
    $args = array_merge($args, $group);
  }
  $join = implode(' ', $join);
  $where = implode(' AND ', $where);

  $sql = db_rewrite_sql(
    "SELECT DISTINCT n.nid, n.sticky, n.title, n.created, p.price
    FROM {node} AS n $join
    WHERE $where
    ORDER BY $order"
  );
  $sql_count = db_rewrite_sql(
    "SELECT COUNT(DISTINCT n.nid)
    FROM {node} AS n $join
    WHERE $where"
  );
  return pager_query(
    $sql, variable_get('default_nodes_main', 10), $pager, $sql_count, $args
  );
}
